@@ -7,13 +7,16 @@ the complex refractive indices in the field of view.
 from __future__ import annotations
 import dataclasses
 import logging
+import typing
 
 import cupy as cp
 import cupyx.scipy.ndimage
 import numpy as np
+import numpy.typing as npt
 import scipy.interpolate
 
 import tike.linalg
+import tike.precision
 
 logger = logging.getLogger(__name__)
 
@@ -39,21 +42,31 @@ class ObjectOptions:
     mdecay: float = 0.9
     """The proportion of the first moment that is previous first moments."""
 
-    v: np.array = dataclasses.field(init=False, default_factory=lambda: None)
+    v: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """The second moment for adaptive moment."""
 
-    m: np.array = dataclasses.field(init=False, default_factory=lambda: None)
+    m: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """The first moment for adaptive moment."""
 
-    preconditioner: np.array = dataclasses.field(init=False,
-                                                 default_factory=lambda: None)
+    preconditioner: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """The magnitude of the illumination used for conditioning the object updates."""
 
-    combined_update: np.array = dataclasses.field(init=False,
-                                                  default_factory=lambda: None)
+    combined_update: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """Used for compact batch updates."""
 
-    clip_magnitude: bool = True
+    clip_magnitude: bool = False
     """Whether to force the object magnitude to remain <= 1."""
 
     def copy_to_device(self, comm):
@@ -126,7 +139,8 @@ def smoothness_constraint(x, a):
     """
     if 0 <= a and a < 1.0 / 8.0:
         logger.info("Object smooth constrained with kernel param %.3e", a)
-        weights = cp.ones([1] * (x.ndim - 2) + [3, 3], dtype='float32') * a
+        weights = cp.ones([1] * (x.ndim - 2) + [3, 3],
+                          dtype=tike.precision.floating) * a
         weights[..., 1, 1] = 1.0 - 8.0 * a
         x.real = cupyx.scipy.ndimage.convolve(x.real, weights, mode='nearest')
         x.imag = cupyx.scipy.ndimage.convolve(x.imag, weights, mode='nearest')
@@ -136,7 +150,7 @@ def smoothness_constraint(x, a):
             f"Smoothness constraint must be in range [0, 1/8) not {a}.")
 
 
-def get_padded_object(scan, probe):
+def get_padded_object(scan, probe, extra: int = 0):
     """Return a ones-initialized object and shifted scan positions.
 
     An complex object array is initialized with shape such that the area
@@ -144,20 +158,15 @@ def get_padded_object(scan, probe):
     positions are shifted to be centered in this newly initialized object
     array.
     """
-    pad = probe.shape[-1] // 2
-    # Shift scan positions to zeros
-    scan = scan - np.min(scan, axis=-2) + pad
-
-    span = np.max(scan[..., 0]), np.max(scan[..., 1])
-
-    height = probe.shape[-1] + int(span[0]) + pad
-    width = probe.shape[-1] + int(span[1]) + pad
-
-    return np.full(
-        shape=(height, width),
-        dtype='complex64',
-        fill_value=np.complex64(0.5 + 0j),
-    ), scan
+    int_scan = scan // 1
+    min_corner = np.min(int_scan, axis=-2)
+    max_corner = np.max(int_scan, axis=-2)
+    span = max_corner - min_corner + probe.shape[-1] + 2 + 2 * extra
+    return np.full_like(
+        probe,
+        shape=span.astype(tike.precision.integer),
+        fill_value=tike.precision.cfloating(0.5 + 0j),
+    ), scan + 1 - min_corner + extra
 
 
 def _int_min_max(x):
@@ -177,7 +186,7 @@ def get_absorbtion_image(data, scan, *, rescale=1.0, method='cubic'):
     ----------
     data : (FRAME, WIDE, HIGH)
         The intensity (square of the absolute value) of the propagated
-        wavefront; i.e. what the detector records. FFT-shifted so the
+        wavefront i.e. what the detector records. FFT-shifted so the
         diffraction peak is at the corners.
     scan : (POSI, 2) float32
         Coordinates of the minimum corner of the probe grid for each
@@ -206,3 +215,17 @@ def get_absorbtion_image(data, scan, *, rescale=1.0, method='cubic'):
         fill_value=np.amax(values),
     )
     return np.reshape(absorption_image, coord0.shape)
+
+
+def remove_object_ambiguity(
+    psi: npt.NDArray[cp.complex64],
+    probe: npt.NDArray[cp.complex64],
+    preconditioner: npt.NDArray[cp.complex64],
+) -> typing.Tuple[npt.NDArray[cp.complex64], npt.NDArray[cp.complex64]]:
+    """Remove normalization ambiguity between probe and psi"""
+    W = preconditioner.real
+    W = W / tike.linalg.mnorm(W)
+    object_norm = 2 * np.sqrt(np.mean(np.square(np.abs(psi)) * W))
+    psi = psi / object_norm
+    probe = probe * object_norm
+    return psi, probe
