@@ -38,13 +38,16 @@ allowed to vary.
 from __future__ import annotations
 import dataclasses
 import logging
+import typing
 
 import cupy as cp
 import cupyx.scipy.ndimage
 import numpy as np
+import numpy.typing as npt
 
 import tike.linalg
 import tike.random
+import tike.precision
 
 logger = logging.getLogger(__name__)
 
@@ -53,14 +56,14 @@ logger = logging.getLogger(__name__)
 class ProbeOptions:
     """Manage data and setting related to probe correction."""
 
-    orthogonality_constraint: bool = True
+    force_orthogonality: bool = False
     """Forces probes to be orthogonal each iteration."""
 
-    centered_intensity_constraint: bool = False
+    force_centered_intensity: bool = False
     """Forces the probe intensity to be centered."""
 
-    sparsity_constraint: float = 1
-    """Forces a maximum proportion of non-zero elements."""
+    force_sparsity: float = 0.0
+    """Forces this proportion of zero elements."""
 
     use_adaptive_moment: bool = False
     """Whether or not to use adaptive moment."""
@@ -71,10 +74,16 @@ class ProbeOptions:
     mdecay: float = 0.9
     """The proportion of the first moment that is previous first moments."""
 
-    v: np.array = dataclasses.field(init=False, default_factory=lambda: None)
+    v: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """The second moment for adaptive moment."""
 
-    m: np.array = dataclasses.field(init=False, default_factory=lambda: None)
+    m: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
     """The first moment for adaptive moment."""
 
     probe_support: float = 0.0
@@ -97,7 +106,7 @@ class ProbeOptions:
     Degree > 1 is more like a top-hat than a gaussian.
     """
 
-    additional_probe_penalty: float = 1.0
+    additional_probe_penalty: float = 0.0
     """Penalty applied to the last probe for existing.
 
     This penalty encourages the probe energy to concentrate in the lower order
@@ -105,16 +114,35 @@ class ProbeOptions:
     linearly to this value. For example, for three probes, the penalties aplied
     are [0.0, 0.5, 1.0].
 
-    This is a soft constraint as opposed to `sparsity_constraint` which is a
+    This is a soft constraint as opposed to `force_sparsity` which is a
     hard constraint.
     """
 
-    def copy_to_device(self):
+    probe_update_sum: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
+    """Used for momentum updates."""
+
+    preconditioner: typing.Union[npt.NDArray, None] = dataclasses.field(
+        init=False,
+        default_factory=lambda: None,
+    )
+
+    power: typing.List[typing.List[float]] = dataclasses.field(
+        init=False,
+        default_factory=list,
+    )
+    """The power of the primary probe modes at each iteration."""
+
+    def copy_to_device(self, comm):
         """Copy to the current GPU memory."""
         if self.v is not None:
             self.v = cp.asarray(self.v)
         if self.m is not None:
             self.m = cp.asarray(self.m)
+        if self.preconditioner is not None:
+            self.preconditioner = comm.pool.bcast([self.preconditioner])
         return self
 
     def copy_to_host(self):
@@ -123,14 +151,16 @@ class ProbeOptions:
             self.v = cp.asnumpy(self.v)
         if self.m is not None:
             self.m = cp.asnumpy(self.m)
+        if self.preconditioner is not None:
+            self.preconditioner = cp.asnumpy(self.preconditioner[0])
         return self
 
     def resample(self, factor: float) -> ProbeOptions:
         """Return a new `ProbeOptions` with the parameters rescaled."""
         return ProbeOptions(
-            orthogonality_constraint=self.orthogonality_constraint,
-            centered_intensity_constraint=self.centered_intensity_constraint,
-            sparsity_constraint=self.sparsity_constraint,
+            force_orthogonality=self.force_orthogonality,
+            force_centered_intensity=self.force_centered_intensity,
+            force_sparsity=self.force_sparsity,
             use_adaptive_moment=self.use_adaptive_moment,
             vdecay=self.vdecay,
             mdecay=self.mdecay,
@@ -375,17 +405,10 @@ def update_eigen_probe(
         c=c,
         m=m,
     )
-    if comm.use_mpi:
-        update[0] = comm.Allreduce_mean(
-            update,
-            axis=-5,
-        )
-        update = comm.pool.bcast([update[0]])
-    else:
-        update = comm.pool.bcast([comm.pool.reduce_mean(
-            update,
-            axis=-5,
-        )])
+    update = comm.pool.bcast([comm.Allreduce_mean(
+        update,
+        axis=-5,
+    )])
 
     (eigen_probe, n, d, d_mean) = (list(a) for a in zip(*comm.pool.map(
         _get_d,
@@ -398,17 +421,10 @@ def update_eigen_probe(
         m=m,
     )))
 
-    if comm.use_mpi:
-        d_mean[0] = comm.Allreduce_mean(
-            d_mean,
-            axis=-3,
-        )
-        d_mean = comm.pool.bcast([d_mean[0]])
-    else:
-        d_mean = comm.pool.bcast([comm.pool.reduce_mean(
-            d_mean,
-            axis=-3,
-        )])
+    d_mean = comm.pool.bcast([comm.Allreduce_mean(
+        d_mean,
+        axis=-3,
+    )])
 
     weights = list(
         comm.pool.map(
@@ -461,8 +477,10 @@ def add_modes_random_phase(probe, nmodes):
     Brocklesby, "Ptychographic coherent diffractive imaging with orthogonal
     probe relaxation." Opt. Express 24, 8360 (2016). doi: 10.1364/OE.24.008360
     """
-    all_modes = np.empty((*probe.shape[:-3], nmodes, *probe.shape[-2:]),
-                         dtype='complex64')
+    all_modes = np.empty_like(
+        probe,
+        shape=(*probe.shape[:-3], nmodes, *probe.shape[-2:]),
+    )
     pw = probe.shape[-1]
     for m in range(nmodes):
         if m < probe.shape[-3]:
@@ -583,7 +601,7 @@ def add_modes_cartesian_hermite(probe, nmodes: int):
             new_probes.append(basis)
 
             if len(new_probes) == nmodes:
-                return np.concatenate(new_probes, axis=-3)
+                return np.concatenate(new_probes, axis=-3)[..., :nmodes, :, :]
 
     raise RuntimeError(
         "`add_modes_cartesian_hermite` never reached a return statement."
@@ -647,7 +665,7 @@ def init_varying_probe(
         *scan.shape[:-1],
         num_eigen_probes,
         shared_probe.shape[-3],
-    ).astype('float32')
+    ).astype(tike.precision.floating)
     weights -= np.mean(weights, axis=-3, keepdims=True)
     # The weight of the first eigen probe is non-zero.
     weights[..., 0, :] = 1.0
@@ -662,14 +680,14 @@ def init_varying_probe(
         num_eigen_probes - 1,
         probes_with_modes,
         *shared_probe.shape[-2:],
-    ).astype('complex64')
+    )
     # The eigen probes are mean normalized.
     eigen_probe /= tike.linalg.mnorm(eigen_probe, axis=(-2, -1), keepdims=True)
 
     return eigen_probe, weights
 
 
-def orthogonalize_eig(x):
+def orthogonalize_eig(x: npt.NDArray,) -> typing.Tuple[npt.NDArray, npt.NDArray]:
     """Orthogonalize modes of x using eigenvectors of the pairwise dot product.
 
     Parameters
@@ -677,6 +695,13 @@ def orthogonalize_eig(x):
     x : (..., nmodes, :, :) array_like complex64
         An array of the probe modes vectorized
 
+    Returns
+    -------
+    x : array_like
+        The orthogonalized probes
+    power : array_like
+        The power of each probe
+
     References
     ----------
     M. Odstrcil, P. Baksh, S. A. Boden, R. Card, J. E. Chad, J. G. Frey, W. S.
@@ -688,7 +713,7 @@ def orthogonalize_eig(x):
     # 'A' holds the dot product of all possible mode pairs. This is equivalent
     # to x^H @ x. We only fill the upper half of `A` because it is
     # conjugate-symmetric.
-    A = xp.empty((*x.shape[:-3], nmodes, nmodes), dtype='complex64')
+    A = xp.empty_like(x, shape=(*x.shape[:-3], nmodes, nmodes))
     for i in range(nmodes):
         for j in range(i, nmodes):
             A[..., i, j] = xp.sum(
@@ -698,16 +723,13 @@ def orthogonalize_eig(x):
     # We find the eigen vectors of x^H @ x in order to get v^H from SVD of x
     # without computing u, s.
     val, vectors = xp.linalg.eigh(A, UPLO='U')
-    # np.linalg.eigh guarantees that the eigen values are returned in ascending
-    # order, so we just reverse the order of modes to have them sorted in
-    # descending order.
-    vectors = vectors[..., ::-1].swapaxes(-1, -2)
-    result = (vectors @ x.reshape(*x.shape[:-2], -1)).reshape(*x.shape)
-    assert np.all(
-        np.diff(tike.linalg.norm(result, axis=(-2, -1), keepdims=False),
-                axis=-1) <= 0
-    ), f"Power of the orthogonalized probes should be monotonically decreasing! {val}"
-    return result
+    result = (vectors.swapaxes(-1, -2) @ x.reshape(*x.shape[:-2], -1)).reshape(
+        *x.shape)
+    power = np.square(tike.linalg.norm(result, axis=(-2, -1), keepdims=False)).flatten()
+    order = np.argsort(power, axis=None, kind='stable')[::-1]
+    result = result[..., order, :, :]
+    power = power[order]
+    return result, power
 
 
 def gaussian(size, rin=0.8, rout=1.0):
@@ -735,7 +757,7 @@ def gaussian(size, rin=0.8, rout=1.0):
     rs = np.sqrt((r - size / 2)**2 + (c - size / 2)**2)
     rmax = np.sqrt(2) * 0.5 * rout * rs.max() + 1.0
     rmin = np.sqrt(2) * 0.5 * rin * rs.max()
-    img = np.zeros((size, size), dtype='float32')
+    img = np.zeros((size, size), dtype=tike.precision.floating)
     img[rs < rmin] = 1.0
     img[rs > rmax] = 0.0
     zone = np.logical_and(rs > rmin, rs < rmax)
@@ -775,10 +797,10 @@ def constrain_center_peak(probe):
 
 
 def constrain_probe_sparsity(probe, f):
-    """Constrain the probe intensity so no more than f/1 elements are nonzero."""
-    if f == 1:
+    """Constrain the probe intensity so at least `f` fraction elements are zero."""
+    if f == 0:
         return probe
-    logger.info("Constrained probe intensity spasity to %.3e", f)
+    logger.info("Constrained probe intensity so %.3e percent are zero", f * 100)
     # First reshape the probe to 3D so it is a single stack of 2D images.
     stack = probe.reshape((-1, *probe.shape[-2:]))
     intensity = np.sum(np.square(np.abs(stack)), axis=0)
@@ -789,7 +811,7 @@ def constrain_probe_sparsity(probe, f):
         mode='wrap',
     )
     # Get the coordinates of the smallest k values
-    k = int((1 - f) * probe.shape[-1] * probe.shape[-2])
+    k = int(f * probe.shape[-1] * probe.shape[-2])
     smallest = np.argpartition(intensity, k, axis=None)[:k]
     coords = cp.unravel_index(smallest, dims=probe.shape[-2:])
     # Set these k smallest values to zero in all probes
@@ -833,14 +855,14 @@ def finite_probe_support(probe, *, radius=0.5, degree=5, p=1.0):
     centers = cp.linspace(-0.5, 0.5, num=N, endpoint=False) + 0.5 / N
     i, j = cp.meshgrid(centers, centers)
     mask = 1 - cp.exp(-(cp.square(i / radius) + cp.square(j / radius))**degree)
-    return p * mask.astype('float32')
+    return p * mask.astype(tike.precision.floating)
 
 
 if __name__ == "__main__":
     cp.random.seed()
     x = (cp.random.rand(7, 1, 9, 3, 3) +
-         1j * cp.random.rand(7, 1, 9, 3, 3)).astype('complex64')
-    x1 = orthogonalize_eig(x)
+         1j * cp.random.rand(7, 1, 9, 3, 3)).astype(tike.precision.cfloating)
+    x1, _ = orthogonalize_eig(x)
     assert x1.shape == x.shape, x1.shape
 
     p = (cp.random.rand(3, 7, 7) * 100).astype(int)
