@@ -57,20 +57,21 @@ __all__ = [
 ]
 
 import copy
-from itertools import product
 import logging
 import time
 import typing
 import warnings
 
 import numpy as np
+import numpy.typing as npt
 import cupy as cp
 
-from tike.operators import Ptycho
-from tike.communicators import Comm, MPIComm
+import tike.operators
+import tike.communicators
 import tike.opt
 from tike.ptycho import solvers
-import tike.random
+import tike.cluster
+import tike.precision
 
 from .position import (
     PositionOptions,
@@ -157,25 +158,26 @@ def simulate(
 
     """
     check_allowed_positions(scan, psi, probe.shape)
-    with Ptycho(
+    with tike.operators.Ptycho(
             probe_shape=probe.shape[-1],
             detector_shape=int(detector_shape),
             nz=psi.shape[-2],
             n=psi.shape[-1],
             **kwargs,
     ) as operator:
-        scan = operator.asarray(scan, dtype='float32')
-        psi = operator.asarray(psi, dtype='complex64')
-        probe = operator.asarray(probe, dtype='complex64')
+        scan = operator.asarray(scan, dtype=tike.precision.floating)
+        psi = operator.asarray(psi, dtype=tike.precision.cfloating)
+        probe = operator.asarray(probe, dtype=tike.precision.cfloating)
         if eigen_weights is not None:
-            eigen_weights = operator.asarray(eigen_weights, dtype='float32')
+            eigen_weights = operator.asarray(eigen_weights,
+                                             dtype=tike.precision.floating)
         data = _compute_intensity(operator, psi, scan, probe, eigen_weights,
                                   eigen_probe, fly)
         return operator.asnumpy(data.real)
 
 
 def reconstruct(
-    data: np.typing.NDArray,
+    data: npt.NDArray,
     parameters: solvers.PtychoParameters,
     model: str = 'gaussian',
     num_gpu: typing.Union[int, typing.Tuple[int, ...]] = 1,
@@ -217,7 +219,7 @@ def reconstruct(
 
     .. seealso:: :py:func:`tike.ptycho.ptycho.reconstruct_multigrid`, :py:func:`tike.ptycho.ptycho.Reconstruction`
     """
-    with tike.ptycho.Reconstruction(
+    with Reconstruction(
             data,
             parameters,
             model,
@@ -273,7 +275,7 @@ class Reconstruction():
 
     def __init__(
         self,
-        data: np.typing.NDArray,
+        data: npt.NDArray,
         parameters: solvers.PtychoParameters,
         model: str = 'gaussian',
         num_gpu: typing.Union[int, typing.Tuple[int, ...]] = 1,
@@ -304,29 +306,29 @@ class Reconstruction():
                         parameters.algorithm_options.num_iter,
                     ))
 
-        if use_mpi is True:
-            mpi = MPIComm
+        if use_mpi:
+            mpi = tike.communicators.MPIComm
             if parameters.psi is None:
                 raise ValueError(
                     "When MPI is enabled, initial object guess cannot be None; "
                     "automatic psi initialization is not synchronized "
                     "across processes.")
         else:
-            mpi = None
+            mpi = tike.communicators.NoMPIComm
 
         self.data = data
         self.parameters = parameters
         self._device_parameters = copy.deepcopy(parameters)
         self.device = cp.cuda.Device(
             num_gpu[0] if isinstance(num_gpu, tuple) else None)
-        self.operator = Ptycho(
+        self.operator = tike.operators.Ptycho(
             probe_shape=parameters.probe.shape[-1],
             detector_shape=data.shape[-1],
             nz=parameters.psi.shape[-2],
             n=parameters.psi.shape[-1],
             model=model,
         )
-        self.comm = Comm(num_gpu, mpi)
+        self.comm = tike.communicators.Comm(num_gpu, mpi)
 
     def __enter__(self):
         self.device.__enter__()
@@ -344,36 +346,40 @@ class Reconstruction():
             self._device_parameters.scan,
             self.data,
             self._device_parameters.eigen_weights,
-        ) = split_by_scan_grid(
+        ) = tike.cluster.by_scan_grid(
             self.comm.pool,
             (
                 self.comm.pool.num_workers
                 if odd_pool else self.comm.pool.num_workers // 2,
                 1 if odd_pool else 2,
             ),
-            ('float32', 'uint16', 'float32'),
+            (tike.precision.floating, tike.precision.floating
+             if self.data.itemsize > 2 else self.data.dtype,
+             tike.precision.floating),
             self._device_parameters.scan,
             self.data,
             self._device_parameters.eigen_weights,
         )
 
         self._device_parameters.psi = self.comm.pool.bcast(
-            [self._device_parameters.psi.astype('complex64')])
+            [self._device_parameters.psi.astype(tike.precision.cfloating)])
 
         self._device_parameters.probe = self.comm.pool.bcast(
-            [self._device_parameters.probe.astype('complex64')])
+            [self._device_parameters.probe.astype(tike.precision.cfloating)])
 
         if self._device_parameters.probe_options is not None:
             self._device_parameters.probe_options = self._device_parameters.probe_options.copy_to_device(
-            )
+                self.comm,)
 
         if self._device_parameters.object_options is not None:
             self._device_parameters.object_options = self._device_parameters.object_options.copy_to_device(
                 self.comm,)
 
         if self._device_parameters.eigen_probe is not None:
-            self._device_parameters.eigen_probe = self.comm.pool.bcast(
-                [self._device_parameters.eigen_probe.astype('complex64')])
+            self._device_parameters.eigen_probe = self.comm.pool.bcast([
+                self._device_parameters.eigen_probe.astype(
+                    tike.precision.cfloating)
+            ])
 
         if self._device_parameters.position_options is not None:
             # TODO: Consider combining put/split, get/join operations?
@@ -385,7 +391,7 @@ class Reconstruction():
 
         # Unique batch for each device
         self.batches = self.comm.pool.map(
-            getattr(tike.random,
+            getattr(tike.cluster,
                     self._device_parameters.algorithm_options.batch_method),
             self._device_parameters.scan,
             num_cluster=self._device_parameters.algorithm_options.num_batch,
@@ -413,19 +419,29 @@ class Reconstruction():
                 f"{len(self._device_parameters.algorithm_options.times):,d}")
 
             if self._device_parameters.probe_options is not None:
-                if self._device_parameters.probe_options.centered_intensity_constraint:
+                if self._device_parameters.probe_options.force_centered_intensity:
                     self._device_parameters.probe = self.comm.pool.map(
                         constrain_center_peak,
                         self._device_parameters.probe,
                     )
-                if self._device_parameters.probe_options.sparsity_constraint < 1:
+                if self._device_parameters.probe_options.force_sparsity < 1:
                     self._device_parameters.probe = self.comm.pool.map(
                         constrain_probe_sparsity,
                         self._device_parameters.probe,
                         f=self._device_parameters.probe_options
-                        .sparsity_constraint,
+                        .force_sparsity,
                     )
 
+                if self._device_parameters.probe_options.force_orthogonality:
+                    (
+                        self._device_parameters.probe,
+                        power,
+                    ) = (list(a) for a in zip(*self.comm.pool.map(
+                        tike.ptycho.probe.orthogonalize_eig,
+                        self._device_parameters.probe,
+                    )))
+                    self._device_parameters.probe_options.power.append(power[0].get())
+
             self._device_parameters = getattr(
                 solvers,
                 self._device_parameters.algorithm_options.name,
@@ -448,16 +464,12 @@ class Reconstruction():
                     and self._device_parameters.position_options[0]
                     .use_position_regularization):
 
-                # TODO: Regularize on all GPUs
-                self._device_parameters.scan[
-                    0], _ = affine_position_regularization(
-                        self.operator,
-                        self._device_parameters.psi[0],
-                        self._device_parameters.probe[0],
-                        self._device_parameters.position_options
-                        .initial_scan[0],
-                        self._device_parameters.scan[0],
-                    )
+                (self._device_parameters.position_options
+                ) = affine_position_regularization(
+                    self.comm,
+                    updated=self._device_parameters.scan,
+                    position_options=self._device_parameters.position_options,
+                )
 
             self._device_parameters.algorithm_options.times.append(
                 time.perf_counter() - start)
@@ -559,8 +571,8 @@ class Reconstruction():
 
     def append_new_data(
         self,
-        new_data: np.typing.NDArray,
-        new_scan: np.typing.NDArray,
+        new_data: npt.NDArray,
+        new_scan: npt.NDArray,
     ) -> None:
         """Append new diffraction patterns and positions to existing result."""
         # Assign positions and data to correct devices.
@@ -573,14 +585,14 @@ class Reconstruction():
             order,
             new_scan,
             new_data,
-        ) = split_by_scan_grid(
+        ) = tike.cluster.by_scan_grid(
             self.comm.pool,
             (
                 self.comm.pool.num_workers
                 if odd_pool else self.comm.pool.num_workers // 2,
                 1 if odd_pool else 2,
             ),
-            ('float32', 'uint16'),
+            (self._device_parameters.scan[0].dtype, self.data[0].dtype),
             new_scan,
             new_data,
         )
@@ -606,7 +618,7 @@ class Reconstruction():
 
         # Rebatch on each device
         self.batches = self.comm.pool.map(
-            getattr(tike.random,
+            getattr(tike.cluster,
                     self._device_parameters.algorithm_options.batch_method),
             self._device_parameters.scan,
             num_cluster=self._device_parameters.algorithm_options.num_batch,
@@ -638,7 +650,7 @@ def _order_join(a, b):
 
 def _get_rescale(data, psi, scan, probe, num_batch, operator):
 
-    n = cp.zeros(2, dtype='float64')
+    n = cp.zeros(2, dtype=np.double)
 
     for b in tike.opt.batch_indicies(data.shape[-3],
                                      num_batch,
@@ -679,10 +691,7 @@ def _rescale_probe(operator, comm, data, psi, scan, probe, num_batch):
             "Increase num_batch to process your data in smaller chunks "
             "or use CuPy to switch to the Unified Memory Pool.")
 
-    if comm.use_mpi:
-        n = np.sqrt(comm.Allreduce_reduce(n, 'cpu'))
-    else:
-        n = np.sqrt(comm.reduce(n, 'cpu'))
+    n = np.sqrt(comm.Allreduce_reduce_cpu(n))
 
     rescale = cp.asarray(n[0] / n[1])
 
@@ -693,128 +702,14 @@ def _rescale_probe(operator, comm, data, psi, scan, probe, num_batch):
     return comm.pool.bcast([probe[0]])
 
 
-def _split(m, x, dtype):
-    return cp.asarray(x[m], dtype=dtype)
-
-
-def split_by_scan_grid(pool, shape, dtype, scan, *args, fly=1):
-    """Split the field of view into a 2D grid.
-
-    Mask divide the data into a 2D grid of spatially contiguous regions.
-
-    Parameters
-    ----------
-    shape : tuple of int
-        The number of grid divisions along each dimension.
-    dtype : List[str]
-        The datatypes of the args after splitting.
-    scan : (nscan, 2) float32
-        The 2D coordinates of the scan positions.
-    args : (nscan, ...) float32 or None
-        The arrays to be split by scan position.
-    fly : int
-        The number of scan positions per frame.
-
-    Returns
-    -------
-    order : List[array[int]]
-        The locations of the inputs in the original arrays.
-    scan : List[array[float32]]
-        The divided 2D coordinates of the scan positions.
-    args : List[array[float32]] or None
-        Each input divided into regions or None if arg was None.
-    """
-    if len(shape) != 2:
-        raise ValueError('The grid shape must have two dimensions.')
-    vstripes = split_by_scan_stripes(scan, shape[0], axis=0, fly=fly)
-    hstripes = split_by_scan_stripes(scan, shape[1], axis=1, fly=fly)
-    mask = [np.logical_and(*pair) for pair in product(vstripes, hstripes)]
-
-    order = np.arange(scan.shape[-2])
-    order = [order[m] for m in mask]
-
-    split_args = []
-    for arg, t in zip([scan, *args], dtype):
-        if arg is None:
-            split_args.append(None)
-        else:
-            split_args.append(pool.map(_split, mask, x=arg, dtype=t))
-
-    return (order, *split_args)
-
-
-def split_by_scan_stripes(scan, n, fly=1, axis=0):
-    """Return `n` boolean masks that split the field of view into stripes.
-
-    Mask divide the data into spatially contiguous regions along the position
-    axis.
-
-    Split scan into three stripes:
-    >>> [scan[s] for s in split_by_scan_stripes(scan, 3)]
-
-    FIXME: Only uses the first view to divide the positions. Assumes the
-    positions on all angles are distributed similarly.
-
-    Parameters
-    ----------
-    scan : (nscan, 2) float32
-        The 2D coordinates of the scan positions.
-    n : int
-        The number of stripes.
-    fly : int
-        The number of scan positions per frame.
-    axis : int (0 or 1)
-        Which spatial dimension to divide along. i.e. horizontal or vertical.
-
-    Returns
-    -------
-    mask : list of (nscan, ) boolean
-        A list of boolean arrays which divide the scan positions into `n`
-        stripes.
-
-    """
-    if scan.ndim != 2:
-        raise ValueError('scan must have two dimensions.')
-    if n < 1:
-        raise ValueError('The number of stripes must be > 0.')
-
-    nscan, _ = scan.shape
-    if (nscan // fly) * fly != nscan:
-        raise ValueError('The number of scan positions must be an '
-                         'integer multiple of the number of fly positions.')
-
-    # Reshape scan so positions in the same fly scan are not separated
-    scan = scan.reshape(nscan // fly, fly, 2)
-
-    # Determine the edges of the horizontal stripes
-    edges = np.linspace(
-        scan[..., axis].min(),
-        scan[..., axis].max(),
-        n + 1,
-        endpoint=True,
-    )
-
-    # Move the outer edges to include all points
-    edges[0] -= 1
-    edges[-1] += 1
-
-    # Generate masks which put points into stripes
-    return [
-        np.logical_and(
-            edges[i] < scan[:, 0, axis],
-            scan[:, 0, axis] <= edges[i + 1],
-        ).repeat(fly) for i in range(n)
-    ]
-
-
 def reconstruct_multigrid(
-    data: np.typing.NDArray,
+    data: npt.NDArray,
     parameters: solvers.PtychoParameters,
     model: str = 'gaussian',
     num_gpu: typing.Union[int, typing.Tuple[int, ...]] = 1,
     use_mpi: bool = False,
     num_levels: int = 3,
-    interp = None,
+    interp=None,
 ) -> solvers.PtychoParameters:
     """Solve the ptychography problem using a multi-grid method.
 
