@@ -87,11 +87,7 @@ def dm(
             op=op,
         ))
 
-        if comm.use_mpi:
-            # TODO: This reduction should be mean
-            batch_cost.append(comm.Allreduce_reduce(cost, 'cpu'))
-        else:
-            batch_cost.append(comm.reduce(cost, 'cpu'))
+        batch_cost.append(comm.Allreduce_mean(cost, axis=None).get())
 
         (
             psi_update_numerator,
@@ -264,15 +260,10 @@ def _apply_update(
 ):
 
     if recover_psi:
-        if comm.use_mpi:
-            psi_update_numerator = comm.Allreduce_reduce(
-                psi_update_numerator, 'gpu')[0]
-            psi_update_denominator = comm.Allreduce_reduce(
-                psi_update_denominator, 'gpu')[0]
-        else:
-            psi_update_numerator = comm.reduce(psi_update_numerator, 'gpu')[0]
-            psi_update_denominator = comm.reduce(psi_update_denominator,
-                                                 'gpu')[0]
+        psi_update_numerator = comm.Allreduce_reduce_gpu(
+            psi_update_numerator)[0]
+        psi_update_denominator = comm.Allreduce_reduce_gpu(
+            psi_update_denominator)[0]
 
         new_psi = psi_update_numerator / (psi_update_denominator + 1e-9)
         if object_options.use_adaptive_moment:
@@ -291,16 +282,10 @@ def _apply_update(
         psi = comm.pool.bcast([new_psi])
 
     if recover_probe:
-        if comm.use_mpi:
-            probe_update_numerator = comm.Allreduce_reduce(
-                probe_update_numerator, 'gpu')[0]
-            probe_update_denominator = comm.Allreduce_reduce(
-                probe_update_denominator, 'gpu')[0]
-        else:
-            probe_update_numerator = comm.reduce(probe_update_numerator,
-                                                 'gpu')[0]
-            probe_update_denominator = comm.reduce(probe_update_denominator,
-                                                   'gpu')[0]
+        probe_update_numerator = comm.Allreduce_reduce_gpu(
+            probe_update_numerator)[0]
+        probe_update_denominator = comm.Allreduce_reduce_gpu(
+            probe_update_denominator)[0]
 
         new_probe = probe_update_numerator / (probe_update_denominator + 1e-9)
         if probe_options.use_adaptive_moment:
@@ -323,10 +308,7 @@ def _apply_update(
 
 def _get_patches(nearplane, psi, scan, op=None):
     patches = op.diffraction.patch.fwd(
-        patches=cp.zeros(
-            nearplane[..., 0, 0, :, :].shape,
-            dtype='complex64',
-        ),
+        patches=cp.zeros_like(nearplane[..., 0, 0, :, :]),
         images=psi,
         positions=scan,
     )[..., None, None, :, :]
@@ -344,11 +326,9 @@ def _get_nearplane_gradients(
     recover_positions=True,
     op=None,
 ):
-    psi_update_numerator = cp.zeros(psi.shape, dtype='complex64')
-    psi_update_denominator = cp.zeros(psi.shape, dtype='complex64')
-    probe_update_numerator = cp.zeros(probe.shape, dtype='complex64')
-    # position_update_numerator = cp.zeros(scan.shape, dtype='float32')
-    # position_update_denominator = cp.zeros(scan.shape, dtype='float32')
+    psi_update_numerator = cp.zeros_like(psi)
+    psi_update_denominator = cp.zeros_like(psi)
+    probe_update_numerator = cp.zeros_like(probe)
 
     for m in range(probe.shape[-3]):
 
@@ -362,9 +342,6 @@ def _get_nearplane_gradients(
                 positions=scan,
             )
             probe_amp = probe[..., 0, m, :, :] * probe[..., 0, m, :, :].conj()
-            # TODO: Allow this kind of broadcasting inside the patch operator
-            if probe_amp.shape[-3] == 1:
-                probe_amp = cp.tile(probe_amp, (scan.shape[-2], 1, 1))
             psi_update_denominator = op.diffraction.patch.adj(
                 patches=probe_amp,
                 images=psi_update_denominator,
@@ -392,6 +369,4 @@ def _get_nearplane_gradients(
         psi_update_denominator,
         probe_update_numerator,
         probe_update_denominator,
-        # position_update_numerator,
-        # position_update_denominator,
     )
