@@ -6,15 +6,18 @@ __author__ = "Daniel Ching, Viktor Nikitin"
 __copyright__ = "Copyright (c) 2020, UChicago Argonne, LLC."
 __docformat__ = 'restructuredtext en'
 
+import tike.linalg
+import tike.random
+import tike.precision
 
-def random_complex(*args):
-    """Return a complex random array in the range (-0.5, 0.5)."""
-    return (np.random.rand(*args) - 0.5 + 1j * (np.random.rand(*args) - 0.5))
+random_complex = tike.random.numpy_complex
 
 
-def inner_complex(x, y):
-    """Return the complex inner product; the order of the operands matters."""
-    return (x.conj() * y).sum()
+def random_floating(*shape):
+    return tike.random.randomizer_np.random(
+        size=shape,
+        dtype=tike.precision.floating,
+    ) - 0.5
 
 
 class OperatorTests():
@@ -42,13 +45,13 @@ class OperatorTests():
         assert d.shape == self.d.shape, (d.shape, self.d.shape)
         m = self.operator.adj(**{self.d_name: self.d}, **self.kwargs)
         assert m.shape == self.m.shape, (m.shape, self.m.shape)
-        a = inner_complex(d, self.d)
-        b = inner_complex(self.m, m)
+        a = tike.linalg.inner(d, self.d)
+        b = tike.linalg.inner(self.m, m)
         print()
         print('<Fm,   m> = {:.5g}{:+.5g}j'.format(a.real.item(), a.imag.item()))
         print('< d, F*d> = {:.5g}{:+.5g}j'.format(b.real.item(), b.imag.item()))
-        self.xp.testing.assert_allclose(a.real, b.real, rtol=1e-5, atol=0)
-        self.xp.testing.assert_allclose(a.imag, b.imag, rtol=1e-5, atol=0)
+        self.xp.testing.assert_allclose(a.real, b.real, rtol=1e-4, atol=0)
+        self.xp.testing.assert_allclose(a.imag, b.imag, rtol=1e-4, atol=0)
 
     def test_scaled(self):
         """Check that the adjoint operator is scaled."""
@@ -57,13 +60,13 @@ class OperatorTests():
         # all of our operators. Here we only test whether |F*Fm| = |m|.
         d = self.operator.fwd(**{self.m_name: self.m}, **self.kwargs)
         m = self.operator.adj(**{self.d_name: d}, **self.kwargs)
-        a = inner_complex(m, m)
-        b = inner_complex(self.m, self.m)
+        a = tike.linalg.inner(m, m)
+        b = tike.linalg.inner(self.m, self.m)
         print()
         # NOTE: Inner product with self is real-only magnitude of self
         print('<F*Fm, F*Fm> = {:.5g}{:+.5g}j'.format(a.real.item(), 0))
         print('<   m,    m> = {:.5g}{:+.5g}j'.format(b.real.item(), 0))
-        self.xp.testing.assert_allclose(a.real, b.real, rtol=1e-5, atol=0)
+        self.xp.testing.assert_allclose(a.real, b.real, rtol=1e-4, atol=0)
 
     def test_fwd_time(self):
         """Time the forward operation."""
